 	list p=16f914
;**************************************************************
;*	PORTA: 	0 > Ziffer 3	-----+
;*		1 > Ziffer 3	-----+
;*		2 > Ziffer 2	---------------------+
;*		3 > Ziffer 2	---------------------+
;*		4 > Ziffer 1	-------------------------------------+
;*		5 > Ziffer 1	-------------------------------------+
;*		 		     I               I               I               
;*		 		     I               I               I                 
;*		 		     I               I               I                 
;*	PORTE: 	0 > Ziffer 0	-----------------------------------------------------+
;*		1 > Ziffer 0	-----------------------------------------------------+
;*		2 < Poti 		     I               I               I               I  
;*		 		     I               I               I               I  
;*		 		     I               I               I               I  
;*	PORTB:	0 > Segment G	   AAAAA	   AAAAA	   AAAAA	   AAAAA
;*		1 > Segment A	  F     B	  F     B	  F     B	  F     B
;*		2 > Segment F	  F     B	  F     B	  F     B	  F     B
;*		3 > Segment B	   GGGGG  	   GGGGG  	   GGGGG  	   GGGGG  
;*		4 > Segment C	  E     C	  E     C	  E     C	  E     C
;*		5 > Segment E	  E     C	  E     C	  E     C	  E     C
;*		6 > Segment D	   DDDDD  HH	   DDDDD  HH	   DDDDD  HH	   DDDDD  HH
;*		7 > Segment H 
;*				     I               I               I               I
;*	PORTC: 	0 < BCD D00	     I               I               I               I
;*		1 < BCD D01	     I               I               I               I
;*		2 < BCD D02	     I               I               I               I
;*		3 < BCD D03	-----------------------------------------------------A
;*		4 < BCD D20	     I               I               I
;*		5 < BCD D21	     I               I               I
;*		6 < BCD D22	     I               I               I
;*		7 < BCD D23	---------------------A               I
;*				     I                               I
;*	PORTD: 	0 < BCD D10	     I                               I
;*		1 < BCD D11	     I                               I
;*		2 < BCD D12	     I                               I
;*		3 < BCD D13	-------------------------------------A
;*		4 < BCD D30	     I
;*		5 < BCD D31	     I
;*		6 < BCD D32	     I
;*		7 < BCD D33	-----A

;
;; 7-Segment-Tabelle fuer SC39-11
;Segmente
;	addwf	PCL, f
;		  76543210
;		  HDECBFAG
;	retlw	B'01111110'	; 0
;	retlw	B'00011000'	; 1
;	retlw	B'01101011'	; 2
;	retlw	B'01011011'	; 3
;	retlw	B'00011101'	; 4
;		  HDECBFAG
;	retlw	B'01010111'	; 5
;	retlw	B'01110111'	; 6
;	retlw	B'00011010'	; 7
;	retlw	B'01111111'	; 8
;	retlw	B'01011111'	; 9
;		  HDECBFAG
;	retlw	B'00111111'	; A
;	retlw	B'01110101'	; b
;	retlw	B'01100110'	; C
;	retlw	B'01111001'	; d
;	retlw	B'01100111'	; E
;	retlw	B'00100111'	; F
;	retlw	B'00000000'	; dunkel
;	retlw	B'00000001'	; -
;
;
;**************************************************************
;
;sprut (zero) Bredendiek 03/2010
;
; 4-stellige LED-Anzeige mit BCD -Eingang mit 16F914
;
; Prozessor 16F914 o..
; Prozessor-Takt intern 4 MHz
;
; Eingang: 4x4-Bit BCD codiert
; MSD	RD4..RD7
;	RC4..RC7
;	RD0..RD3
; LSD	RC0..RC3
;
; Ausgang: 4x 7-Segment, gemeinsame Kathode, ohne Treibertransistoren
; Typ: Kingbright SC39-11SRWA   SUPER BRIGHT RED (GaAlAs)
; Flussapannung: 1,85 V
;  2 mA ->  2000 cd (ausreichend)
; 20 mA -> 20000 cd
;
; fuer 2 mA average sind 10mA peak noetig
; R = U/I = (5-1.85)/0.01 = 3,15/0,01 = 315 Ohm abzueglich Innenwiderstand des PIC
; zulaessig: 25mA pro Pin ,   90mA gesamt
; 2x25mA / 7 = 7mA
; R = U/I = (5-1.85)/0.007 = 3,15/0,007 = 325 Ohm abzueglich Innenwiderstand des PIC
;
; Segmente: 	RB0..RB6 (RB7)
; Kat MSD	RA0 & RA1
; Kat		RA2 & RA3
; Kat		RA4 & RA5
; Kat LSD	RE0 & RE1
;
;
; Helligkeitsregelung
; Eingang : Spannung an RE2/AN7
; 0V =  50% Helligkeit = 10% DC pro Ziffer
; 5V = 100% Helligkeit = 20% DC pro Ziffer
;
;**************************************************************

; Includedatei fr den 16F914 einbinden

	#include <P16f914.INC>

	ERRORLEVEL      -302    	;SUPPRESS BANK SELECTION MESSAGES

; Configuration festlegen:
; Power on Timer, kein Watchdog, interner-Oscillator, kein Reset-Pin
	__CONFIG	_PWRTE_ON & _WDT_OFF & _INTOSC & _MCLRE_OFF


;**************************************************************
; Variablen festlegen	256 Byte
;  0x20 bis  0x7F	96 Byte
;  0xA0 bis  0xEF	80 Byte
; 0x120 bis 0x16F	80 Byte

w_copy		equ	0x4D	; nur fr INT
s_copy		equ	0x4E	; nur fr INT
p_copy		equ	0x4F	; nur fr INT

; Dezimalstellen
Digit		equ	0x52	; aktuell angezeigte Stelle
Ziffer4		equ	0x53	; Tausender
Ziffer3		equ	0x54	; Hunderter
Ziffer2		equ	0x55	; Zehner
Ziffer1		equ	0x56	; Einer
Ziffer		equ	0x57	; Puffer

Hell		equ	0x58	; Helligkeit

; Konstanten festlegen
#define	Kat4		PORTA,0
#define	Kat3		PORTA,2
#define	Kat2		PORTA,4
#define	Kat1		PORTE,0

#define	Kat4a		PORTA,1
#define	Kat3a		PORTA,3
#define	Kat2a		PORTA,5
#define	Kat1a		PORTE,1

Ini_opt		equ	B'10000100'	; Timer0 int 32:1, pull-up on

;**************************************************************
	org	0
	goto	Init

;**************************************************************
; die Interuptserviceroutine
; wird nicht benoetigt

	org 	4 	
intvec
	movwf	w_copy		; w retten
	swapf	STATUS, w 	; STATUS retten
	clrf	STATUS
	movwf	s_copy		;
	movf	PCLATH, W
	movwf	p_copy
	clrf	PCLATH		; Bank 0 

; Intrupt servic routine


Int_end
	bcf	PIR1,TMR1IF	; Timer1 berlaufflag lschen
	bcf	PIR1,CCP1IF	; Interupt-Flag lschen
	movf	p_copy, W
	movwf	PCLATH
	swapf	s_copy, w	; STATUS zurck
	movwf	STATUS 
	swapf	w_copy, f	; w zurck mit flags
	swapf	w_copy, w

	retfie

;**************************************************************
; 7-Segment-Tabelle fuer SC39-11
Segmente
	addwf	PCL, f
;		  76543210
;		  HDECBFAG
	retlw	B'01111110'	; 0
	retlw	B'00011000'	; 1
	retlw	B'01101011'	; 2
	retlw	B'01011011'	; 3
	retlw	B'00011101'	; 4
	retlw	B'01010111'	; 5
	retlw	B'01110111'	; 6
	retlw	B'00011010'	; 7
	retlw	B'01111111'	; 8
	retlw	B'01011111'	; 9
	retlw	B'00111111'	; A
	retlw	B'01110101'	; b
	retlw	B'01100110'	; C
	retlw	B'01111001'	; d
	retlw	B'01100111'	; E
	retlw	B'00100111'	; F
	retlw	B'00000000'	; dunkel
	retlw	B'00000001'	; -

;**************************************************************
; Routine zum multiplexen Anzeigen am LED-Display
Display
	movlw	B'00111111'	; Ziffern aus
	iorwf	PORTA,f
	movlw	B'00000011'
	iorwf	PORTE,f

	clrf	PORTB		; Segmente aus

	decf	Digit,f		; Ziffernzhler verringern

;anstehende Arbeit:
;Digit=4: BCD einlesen
;Digit=3: anzeigen Ziffer 3     1er
;Digit=2: anzeigen Ziffer 2    10er
;Digit=1: anzeigen Ziffer 1   100er
;Digit=0: anzeigen Ziffer 0  1000er

	btfsc	STATUS, Z
	goto	Disp_0		; Z-Flag=1 ergo Digit=0
	decf	Digit, w
	btfsc	STATUS, Z
	goto	Disp_1		; Digit=1
	btfsc	Digit,2
	goto	Disp_4		; Digit=4
	btfss	Digit,0
	goto	Disp_2		; Digit=2
	goto	Disp_3		; Digit=3

Disp_0
	movlw	5
	movwf	Digit		; Digit wieder auf 5 einstellen

	movfw	Ziffer4		; Wert der 1. Ziffer (1000er)
	bcf	Kat4		; 1. Ziffer (ganz links) einschalten
	bcf	Kat4a
	goto	Disp_end

Disp_1
	movfw	Ziffer3		; Wert der 2. Ziffer (100er)
	bcf	Kat3		; 2. Ziffer einschalten
	bcf	Kat3a
	goto	Disp_end

Disp_2
	movfw	Ziffer2		; Wert der 3. Ziffer (10er)
	bcf	Kat2		; 3. Ziffer einschalten
	bcf	Kat2a
	goto	Disp_end

Disp_3
	movfw	Ziffer1		; Wert der 3. Ziffer (1er)
	bcf	Kat1		; 4. Ziffer (ganz rechts) einschalten
	bcf	Kat1a
	goto	Disp_end

Disp_4
	call	getBCD		; BCD-Eingnge einlesen	
	call	getHell		; Helligkeitspoti auslesen
	goto	Disp_end2	; keine Segmente einschalten

Disp_end
	movwf	Ziffer

Disp_Hell			; hier kann die Helligkeitsregelung rein
	movfw	Hell		; realisiert durch Warten auf einen Mindestwert 0..127 im Timer0
	subwf	TMR0, w		; w = TMR0 - Hell
	bnc	Disp_Hell	; warten bis TMR0>Hell	

	movfw	Ziffer
	call	Segmente	; 7-Segment-Code fr aktuelle Ziffer holen
	movwf	PORTB		; Segmente einschalten
Disp_end2
	return


;**************************************************************
; Das Programm beginnt mit der Initialisierung

Init
	clrf	PORTB		; LEDs aus
	movlw	B'00111111'
	movwf	PORTA
	bsf	PORTE,0
	bsf	PORTE,1
	CLRF	PORTC 		; Init PORTC
	CLRF	PORTD 		; Init PORTD

	bsf     STATUS, RP0	; Bank 1
	movlw   Ini_opt     	; pull-up on; Timer0 32:1 interner Takt
	movwf   OPTION_REG 
	movlw	B'00000000'	; PortB alle output
	movwf	TRISB
	MOVLW	07h 		
	MOVWF	CMCON0 		; alle Comparatoreingnge auf Digital umschalten
	CLRF	ANSEL		; Make all PORTA digital I/O
	movlw	B'11000000'	; PortRA0..5 outputs 
	movwf	TRISA
	movlw	B'11111100'	; PortRE0..1 outputs 
	movwf	TRISE
	MOVLW	0FFh
	MOVWF	TRISC		; Set RC<7:0> as inputs
	MOVWF	TRISD		; Set RD<7:0> as inputs

	bcf     STATUS, RP0	; Bank 0

	BANKSEL	LCDCON 
	CLRF	LCDCON 		; Disable VLCD<3:1>
				; inputs on RC<2:0>

	BANKSEL	PORTB
	clrf	PORTB		; LEDs aus
	movlw	B'00111111'
	movwf	PORTA
	bsf	PORTE,0
	bsf	PORTE,1
	
	clrf	INTCON      	; Interupt disable  


; Anzeige initialisieren
	movlw	4
	movwf	Digit
	movlw	D'17'		; '-'
	movwf	Ziffer1
	movwf	Ziffer2
	movwf	Ziffer3
	movwf	Ziffer4		; Anzeige '----'

; Timer0 initialisieren fr LED-Multiplex-Anzeige
; 16:1 Vorteiler 
; Immer nach 256 x 16 Zyklen (~4 ms) luft der Timer0 ber -> 244 Hz
; da ein Anzeigezyklus aus 5 Schritten besteht, -> 48 Hz Anzeigefrequenz = flackerfrei
	bsf     STATUS, RP0	; auf Bank 1 umschalten
	movlw	B'10000011'	; internen Takt zhlen, Vorteiler zum Timer0, 16:1
	movwf	OPTION_REG
	bcf     STATUS, RP0	; auf Bank 0 zurckschalten
	clrf	TMR0
	bcf	INTCON, T0IE	; Timer0 interupt verbieten
	bcf	INTCON, GIE	; Interupt erlauben
	bcf	INTCON, T0IF	; Flag lschen


; ADC initialisieren fuer Messung an RE2/AN7
	bsf     STATUS, RP0	; auf Bank 1 umschalten
	MOVLW	B'01110000' 	; ADC Frc clock (max 500 kHz)
	MOVWF	ADCON1
	BSF	TRISE,2		; Set RE2 to input
	BSF	ANSEL,7		; Set RE2 to analog
	bcf     STATUS, RP0	; auf Bank 0 zurckschalten

	MOVLW	B'00011101'	; left justify, AN7
	MOVWF	ADCON0		; Vdd Vref, AN7, On
	clrf	Hell		; maximale Helligkeit
;	bsf	ADCON0,GO	; und Start


; endlose Anzeigeschleife
disploop
	btfss	INTCON, T0IF
	goto	disploop
	bcf	INTCON, T0IF
	call	Display
	goto	disploop


;**************************************************************
; BCD-Eingnge auslesen
getBCD
	; 1000er  RD4..RD7
	movfw	PORTD
	andlw	B'11110000'
	movwf	Ziffer4
	bcf	STATUS,C
	rrf	Ziffer4,f
	rrf	Ziffer4,f
	rrf	Ziffer4,f
	rrf	Ziffer4,f

	; 100er  RC4..RC7
	movfw	PORTC
	andlw	B'11110000'
	movwf	Ziffer3
	bcf	STATUS,C
	rrf	Ziffer3,f
	rrf	Ziffer3,f
	rrf	Ziffer3,f
	rrf	Ziffer3,f

	; 10er  RD0..RD3
	movfw	PORTD
	andlw	B'00001111'
	movwf	Ziffer2

	; 1er  RC0..RC3
	movfw	PORTC
	andlw	B'00001111'
	movwf	Ziffer1

	return


;**************************************************************
; Helligkeit bestimmen
getHell	
	BTFSC	ADCON0,GO		; ADC fertig?
	return				; ADC noch nicht fertig
	MOVF	ADRESH,W		; obere 8 Bits lesen,  0..255
	MOVWF	Hell			; 0..255
	comf	Hell, f			; 255..0
	bcf	STATUS,C		; Hell sollte auf <<255 begrenzt werden
;	rrf	Hell,f			; 127..0
	BSF	ADCON0,GO		; Start ADC
	return

;**************************************************************
	end
